@@ -13,11 +13,9 @@
 # ==============================================================================
 
 from datetime import datetime  # To use date in verbose output.
-from typing import Any, Dict, List, Optional, Tuple, Union  # To type Python code (mypy).
+from typing import Any, Dict, List, Optional, Tuple  # To type Python code (mypy).
 
-import numpy as np  # To handle float.
-from numpy import ndarray  # To handle matrix and vectors.
-from scipy.sparse import csr_matrix  # To handle matrix and vectors.
+from scipy.sparse import csr_matrix, vstack  # To handle matrix and vectors.
 from sklearn.metrics import pairwise_distances  # To compute distance.
 
 from cognitivefactory.interactive_clustering.clustering.abstract import (  # To use abstract interface.; To sort clusters after computation.
@@ -126,7 +124,7 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
     def cluster(
         self,
         constraints_manager: AbstractConstraintsManager,
-        vectors: Dict[str, Union[ndarray, csr_matrix]],
+        vectors: Dict[str, csr_matrix],
         nb_clusters: int,
         verbose: bool = False,
         **kargs,
@@ -136,7 +134,7 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
 
         Args:
             constraints_manager (AbstractConstraintsManager): A constraints manager over data IDs that will force clustering to respect some conditions during computation.
-            vectors (Dict[str,Union[ndarray,csr_matrix]]): The representation of data vectors. The keys of the dictionary represents the data IDs. This keys have to refer to the list of data IDs managed by the `constraints_manager`. The value of the dictionary represent the vector of each data. Vectors can be dense (`numpy.ndarray`) or sparse (`scipy.sparse.csr_matrix`).
+            vectors (Dict[str, csr_matrix]): The representation of data vectors. The keys of the dictionary represents the data IDs. This keys have to refer to the list of data IDs managed by the `constraints_manager`. The value of the dictionary represent the vector of each data.
             nb_clusters (int): The number of clusters to compute.  #TODO Set defaults to None with elbow method or other method ?
             verbose (bool, optional): Enable verbose output. Defaults to `False`.
             **kargs (dict): Other parameters that can be used in the clustering.
@@ -161,23 +159,27 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
         # Store `self.vectors`.
         if not isinstance(vectors, dict):
             raise ValueError("The `vectors` parameter has to be a `dict` type.")
-        self.vectors: Dict[str, Union[ndarray, csr_matrix]] = vectors
+        self.vectors: Dict[str, csr_matrix] = vectors
 
         # Store `self.nb_clusters`.
         if nb_clusters < 2:
             raise ValueError("The `nb_clusters` '" + str(nb_clusters) + "' must be greater than or equal to 2.")
         self.nb_clusters: int = nb_clusters
 
-        # Store `self.pairwise_distances_matrix`.
-        self.pairwise_distances_matrix: Dict[str, Dict[str, float]] = {
-            data_ID1: {
-                data_ID2: pairwise_distances(X=self.vectors[data_ID1], Y=self.vectors[data_ID2], metric="euclidean",)[
-                    0
-                ][0].astype(np.float64)
-                for data_ID2 in self.list_of_data_IDs
+        # Compute pairwise distances.
+        matrix_of_pairwise_distances: csr_matrix = pairwise_distances(
+            X=vstack(self.vectors[data_ID] for data_ID in self.constraints_manager.get_list_of_managed_data_IDs()),
+            metric="euclidean",  # TODO get different pairwise_distances config in **kargs
+        )
+
+        # Format pairwise distances in a dictionary and store `self.dict_of_pairwise_distances`.
+        self.dict_of_pairwise_distances: Dict[str, Dict[str, float]] = {
+            vector_ID1: {
+                vector_ID2: float(matrix_of_pairwise_distances[i1, i2])
+                for i2, vector_ID2 in enumerate(self.constraints_manager.get_list_of_managed_data_IDs())
             }
-            for data_ID1 in self.list_of_data_IDs
-        }  # TODO need ?
+            for i1, vector_ID1 in enumerate(self.constraints_manager.get_list_of_managed_data_IDs())
+        }
 
         ###
         ### INITIALIZE HIERARCHICAL CONSTRAINED CLUSTERING
@@ -450,7 +452,7 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
         return parent_cluster_ID
 
     # ==============================================================================
-    # COMPUTE DISTANCE BETWEEN CLUSTERINGNEW ITERATION OF CLUSTERING :
+    # COMPUTE DISTANCE BETWEEN CLUSTERING NEW ITERATION OF CLUSTERING :
     # ==============================================================================
     def _compute_distance(self, cluster_IDi: int, cluster_IDj: int) -> float:
         """
@@ -481,7 +483,7 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
 
             return max(
                 [
-                    self.pairwise_distances_matrix[data_ID_in_cluster_IDi][data_ID_in_cluster_IDj]
+                    self.dict_of_pairwise_distances[data_ID_in_cluster_IDi][data_ID_in_cluster_IDj]
                     for data_ID_in_cluster_IDi in self.clusters_storage[cluster_IDi].members
                     for data_ID_in_cluster_IDj in self.clusters_storage[cluster_IDj].members
                 ]
@@ -489,19 +491,17 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
 
         # Case 2 : `self.linkage` is "average".
         if self.linkage == "average":
-            vector_centroid1 = self.clusters_storage[cluster_IDi].centroid
-            vector_centroid2 = self.clusters_storage[cluster_IDj].centroid
             return pairwise_distances(
-                X=vector_centroid1,
-                Y=vector_centroid2,
+                X=self.clusters_storage[cluster_IDi].centroid,
+                Y=self.clusters_storage[cluster_IDj].centroid,
                 metric="euclidean",  # TODO: Load different parameters for distance computing ?
-            )[0][0].astype(np.float64)
+            )[0][0]
 
         # Case 3 : `self.linkage` is "single".
         if self.linkage == "single":
             return min(
                 [
-                    self.pairwise_distances_matrix[data_ID_in_cluster_IDi][data_ID_in_cluster_IDj]
+                    self.dict_of_pairwise_distances[data_ID_in_cluster_IDi][data_ID_in_cluster_IDj]
                     for data_ID_in_cluster_IDi in self.clusters_storage[cluster_IDi].members
                     for data_ID_in_cluster_IDj in self.clusters_storage[cluster_IDj].members
                 ]
@@ -515,7 +515,7 @@ class HierarchicalConstrainedClustering(AbstractConstrainedClustering):
         )
         return sum(
             [
-                self.pairwise_distances_matrix[data_IDi][data_IDj]
+                self.dict_of_pairwise_distances[data_IDi][data_IDj]
                 for i, data_IDi in enumerate(merged_members)
                 for j, data_IDj in enumerate(merged_members)
                 if i < j
@@ -703,7 +703,7 @@ class Cluster:
     # ==============================================================================
     def __init__(
         self,
-        vectors: Dict[str, Union[ndarray, csr_matrix]],
+        vectors: Dict[str, csr_matrix],
         cluster_ID: int,
         clustering_iteration: int,
         children: Optional[List["Cluster"]] = None,
@@ -713,7 +713,7 @@ class Cluster:
         The constructor for Cluster class.
 
         Args:
-            vectors (Dict[str,Union[ndarray,csr_matrix]]): The representation of data vectors. The keys of the dictionary represents the data IDs. This keys have to refer to the list of data IDs managed by the `constraints_manager` (if `constraints_manager` is set). The value of the dictionary represent the vector of each data. Vectors can be dense (`numpy.ndarray`) or sparse (`scipy.sparse.csr_matrix`).
+            vectors (Dict[str, csr_matrix]): The representation of data vectors. The keys of the dictionary represents the data IDs. This keys have to refer to the list of data IDs managed by the `constraints_manager` (if `constraints_manager` is set). The value of the dictionary represent the vector of each data.
             cluster_ID (int): The cluster ID that is defined during `HierarchicalConstrainedClustering.cluster` running.
             clustering_iteration (int): The cluster iteration that is defined during `HierarchicalConstrainedClustering.cluster` running.
             children (Optional[List["Cluster"]], optional): A list of clusters children for cluster initialization. Incompatible with `members` parameter. Defaults to `None`.
@@ -724,7 +724,7 @@ class Cluster:
         """
 
         # Store links to `vectors`.
-        self.vectors: Dict[str, Union[ndarray, csr_matrix]] = vectors
+        self.vectors: Dict[str, csr_matrix] = vectors
 
         # Cluster ID and Clustering iteration.
         self.cluster_ID: int = cluster_ID
@@ -792,9 +792,7 @@ class Cluster:
         """
 
         # Update centroids.
-        self.centroid: Union[ndarray, csr_matrix] = (
-            sum([self.vectors[data_ID] for data_ID in self.members]) / self.get_cluster_size()
-        )
+        self.centroid: csr_matrix = sum([self.vectors[data_ID] for data_ID in self.members]) / self.get_cluster_size()
 
     # ==============================================================================
     # GET CLUSTER SIZE :
