@@ -12,11 +12,9 @@
 # IMPORT PYTHON DEPENDENCIES
 # ==============================================================================
 
-from typing import Dict, Union  # To type Python code (mypy).
+from typing import Dict  # To type Python code (mypy).
 
-import numpy as np  # To handle float.
 import spacy  # To apply spacy language models.
-from numpy import ndarray  # To handle matrix and vectors.
 from scipy.sparse import csr_matrix  # To handle matrix and vectors.
 from sklearn.feature_extraction.text import TfidfVectorizer  # To apply TF-IDF vectorisation.
 
@@ -28,7 +26,7 @@ def vectorize(
     dict_of_texts: Dict[str, str],
     vectorizer_type: str = "tfidf",
     spacy_language_model: str = "fr_core_news_sm",
-) -> Dict[str, Union[ndarray, csr_matrix]]:
+) -> Dict[str, csr_matrix]:
     """
     A method used to vectorize texts.
     Severals vectorizer are available : TFIDF, spaCy language model.
@@ -48,7 +46,7 @@ def vectorize(
         ValueError: Raises error if `vectorizer_type` is not implemented or if the `spacy_language_model` is not installed.
 
     Returns:
-        Dict[str, Union[ndarray,csr_matrix]]: A dictionary that contains the computed vectors.
+        Dict[str, csr_matrix]: A dictionary that contains the computed vectors.
 
     Examples:
         ```python
@@ -75,7 +73,7 @@ def vectorize(
     """
 
     # Initialize dictionary of vectors.
-    dict_of_vectors: Dict[str, Union[ndarray, csr_matrix]] = {}
+    dict_of_vectors: Dict[str, csr_matrix] = {}
 
     ###
     ### Case of TFIDF vectorization.
@@ -95,9 +93,7 @@ def vectorize(
         )
 
         # Format dictionary of vectors to return.
-        dict_of_vectors = {
-            data_ID: tfidf_vectorization[i].astype(np.float64) for i, data_ID in enumerate(dict_of_texts.keys())
-        }
+        dict_of_vectors = {data_ID: tfidf_vectorization[i] for i, data_ID in enumerate(dict_of_texts.keys())}
 
         # Return the dictionary of vectors.
         return dict_of_vectors
@@ -123,9 +119,7 @@ def vectorize(
             ) from err
 
         # Apply vectorization.
-        dict_of_vectors = {
-            data_ID: spacy_nlp(str(text)).vector.astype(np.float64) for data_ID, text in dict_of_texts.items()
-        }
+        dict_of_vectors = {data_ID: csr_matrix(spacy_nlp(str(text)).vector) for data_ID, text in dict_of_texts.items()}
 
         # Return the dictionary of vectors.
         return dict_of_vectors
